/*
* =====================================================================
*  Filename: ConditionDlg.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#pragma once

#include "afxwin.h"
#include "afxcmn.h"
#include "afxdtctl.h"
#include "NumEdit.h"

class CConditionDlg;

/**
\typedef ATL::IDispEventImpl<1, CConditionDlg, 
                              &__uuidof(_ICQGCELEvents), 
                              &__uuidof(__CQG), 4, 0> ICQGCELDispEventImpl
\brief  Convenient definition of IDispEventImpl instantiation
*/
typedef ATL::IDispEventImpl<1,
                              CConditionDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0
                           >
                           ICQGCELDispEventImpl;

// CConditionDlg dialog
class CConditionDlg : public CDialog, public ICQGCELDispEventImpl
{

public:

   /// \brief CConditionDlg constructor.
   /// \param pParent - parent window
   explicit CConditionDlg(
                  CWnd* pParent = NULL); 

   /// \enum Dialog Data
   enum { IDD = IDD_CONDITION_DIALOG };

   /// \brief Dialog icon.
   HICON m_hIcon;

private:

   /// \brief DDX/DDV support.
   virtual void DoDataExchange(
                  CDataExchange* pDX);

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the framework shall set the input 
   /// \return focus to the first control of the dialog box
   virtual BOOL OnInitDialog();

   /// \brief Override to prevent dialog from closing on Enter
   /// \brief and catch Enters in tag name and tag value edit controls.
   virtual void OnOK();

private:

   /// \brief Prepares the container for painting.
   afx_msg void OnPaint();

   /// \brief The system calls this function to obtain the cursor to
   /// \brief display while the user drags the minimized window.
   afx_msg HCURSOR OnQueryDragIcon();

   /// \brief Used to unadvise from CQGCEL events and shutdown it.
   afx_msg void OnDestroy();

   /// \brief Changes selected condition object, and shows its data.
   afx_msg void OnCbnSelChangeCmbCondition();

   /// \brief Requests condition definitions.
   afx_msg void OnBnClickedBtnRequestDefinition();

   /// \brief Updates definition data and parameters on the form.
   afx_msg void OnCbnSelchangeCmbDefinition();

   /// \brief Opens request settings form to fill up request parameters.
   afx_msg void OnBnClickedBtnRequest();

   /// \brief Removes selected condition from the CQGCEL and the combo box control.
   afx_msg void OnBnClickedBtnRemove();

   /// \brief Removes all condition from the CQGCEL and the combo box control.
   afx_msg void OnBnClickedBtnRemoveAll();

   /// \brief Changes filtering type to indices.
   afx_msg void OnBnClickedRdIndices();

   /// \brief Changes filtering type to dates.
   afx_msg void OnBnClickedRdDate();

   /// \brief Used to increment or decrement start range value for filtering.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnStart(
                  NMHDR *pNMHDR,
                  LRESULT *pResult);

   /// \brief Used to increment or decrement end range value for filtering.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnEnd(
                  NMHDR *pNMHDR,
                  LRESULT *pResult);

   /// \brief Shows filtered condition.
   afx_msg void OnBnClickedBtnShow();

   /// \brief Shows error message.
   afx_msg void OnStnDblclickLblError();

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

public:

   /// \brief This map is used to declare handler function for specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL-4_0.dll and get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(CConditionDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x0000000a, OnDataError)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000003, OnDataConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000029, OnConditionDefinitionsResolved)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x0000002e, OnConditionResolved)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x0000002f, OnConditionAdded)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000030, OnConditionUpdated)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000045, OnConditionInserted)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000046, OnConditionRemoved)
   END_SINK_MAP()

private:

   /// \brief Handle event of some abnormal discrepancy
   /// \brief between data expected and data received from CQG.
   /// \brief This event also can be fired when CQGCEL startup fails.
   /// \param obj - Object where the error occurred
   /// \param errorDescription - string describing the error
   /// \return S_OK
   STDMETHOD(OnDataError)(
                  LPDISPATCH obj,
                  BSTR errorDescription);

   /// \brief Fired when some changes occurred in the connection with the CQG data server.
   /// \param newStatus - new connection status
   /// \return S_OK
   STDMETHOD(OnDataConnectionStatusChanged)(
                  eConnectionStatus newStatus);

   /// \brief Fired when the definitions of conditions are resolved
   /// \brief or when some error occurred during request processing.
   /// \param cqg_condition_definitions - collection of condition definitions
   /// \param cqg_error - CQGError object describing last error occurred during the request processing
   /// \return S_OK
   STDMETHOD(OnConditionDefinitionsResolved)(
                  ICQGConditionDefinitions* cqg_condition_definitions, 
                  ICQGError* cqg_error);

   /// \brief Fired when the condition (CQGCondition) is resolved
   /// \brief or when some error occurred during condition request processing.
   /// \param cqg_condition - reference to resolved CQGCondition
   /// \param cqg_error - CQGError object describing last error occurred during processing the request.
   /// \return S_OK
   STDMETHOD(OnConditionResolved)(
                  ICQGCondition* cqg_condition,
                  ICQGError* cqg_error);

   /// \brief Fired when CQGConditionResult item is added to the end of CQGCondition.
   /// \param cqg_condition - reference to changed CQGCondition
   /// \return S_OK
   STDMETHOD(OnConditionAdded)(
                  ICQGCondition* cqg_condition);

   /// \brief Fired when CQGConditionResult item is updated.
   /// \param cqg_condition - reference to changed CQGCondition
   /// \param index_ - specifies the updated CQGConditionResult index
   /// \return S_OK
   STDMETHOD(OnConditionUpdated)(
                  ICQGCondition* cqg_condition,
                  long index_);

   /// \brief Fired when CQGConditionResult item is inserted.
   /// \param cqg_condition - reference to changed CQGCondition
   /// \param index_ - specifies the inserted CQGConditionResult index
   /// \return S_OK
   STDMETHOD(OnConditionInserted)(
                  ICQGCondition* cqg_condition,
                  long index_);

   /// \brief Fired when CQGConditionResult item is removed.
   /// \param cqg_condition - reference to changed CQGCondition
   /// \param index_ - specifies the removed CQGConditionResult index
   /// \return S_OK
   STDMETHOD(OnConditionRemoved)(
                  ICQGCondition* cqg_condition,
                  long index_);

private:

   /// \enum eConditionsColumn. Represents condition columns in the list.
   enum eRecordsColumns;

   /// \enum eFilterTypes. Filtering types.
   enum eFilterTypes;

   /// \enum eDefintionParametersColumns. Represents definition parameters columns in the list.
   enum eDefintionParametersColumns;

   /// \enum eParametersColumns. Represents parameters columns in the list.
   enum eParametersColumns;

private:

   /// \brief Initializes CQGCEL object, and starts the CQGCEL.
   void InitializeCQGCEL();

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Initializes definition parameters list control's columns and styles.
   void InitializeDefParametersList();

   /// \brief Initializes parameters list control's columns and styles.
   void InitializeParametersList();

   /// \brief Initializes specified list control's columns and styles.
   /// \param listCtrl - list control which columns will be configured
   static void InitializeList(
                  CListCtrl& listCtrl);
   
   /// \brief Clears all records from the specified list.
   /// \param listCtrl - list control which will be cleared
   static void ClearRecords(
                  CListCtrl& listCtrl);

   /// \brief Clears all data connected to condition definitions.
   void ClearDefinitionData();

   /// \brief Clears all data(parameters\outputs\records) from the form.
   void ClearAllData();

   /// \brief Clears request parameters from the form.
   void ClearRequestParams();

   /// \brief Clears output parameters from the form.
   void ClearOutputParams();
   
   /// \brief Clears request tags.
   void ClearTags();

   /// \brief Shows all data(parameters\outputs\records) connected
   /// \brief with selected request.
   void ShowAllData();

   /// \brief Shows given range records in the specified list control, after changes.
   void ShowChangedData();

   /// \brief Shows selected request parameters on the form.
   void ShowRequestParams();

   /// \brief Shows specified request's definition parameters on the form.
   /// \param spConditionRequest - Request which definition parameters will be shown
   void ShowReqDefinitionParameters(
                  const ATL::CComPtr<ICQGConditionRequest>& spConditionRequest);

   /// \brief Shows selected request outputs on the form.
   void ShowOutputParams();

   /// \brief Shows given range records in the specified list control.
   /// \param startIndex - range start index
   /// \param endIndex - range end index
   /// \param listCtrl - list control in which records will be shown
   void ShowRecords(
                  long startIndex,
                  long endIndex,
                  CListCtrl& listCtrl);

   /// \brief Adds record with given index into the end of specified list control.
   /// \param recordIndex - record index which will be added
   /// \param listCtrl - list control in which record will be added
   void AddRecord(
                  long recordIndex,
                  CListCtrl& listCtrl);

   /// \brief Shows record with given index into the specified row of the specified list control.
   /// \param recordIndex - record index which will be shown
   /// \param listCtrl - list control in which record will be shown
   /// \param row - row of the list control in which record will be shown
   void ShowRecord(
                  long recordIndex,
                  CListCtrl& listCtrl,
                  long row);

   /// \brief Gets start and end indices and validates them to show records.
   /// \param startIndex - range start index
   /// \param endIndex - range end index
   /// \return Returns true if range is valid.
   bool GetIndices(
                  long& startIndex,
                  long& endIndex);

   /// \brief Gets start and end indices for selected dates and validates them to show records.
   /// \param startIndex - range start index
   /// \param endIndex - range end index
   /// \return Returns true if range is valid.
   bool GetIndicesForDates(
                  long& startIndex,
                  long& endIndex);

   /// \brief Clears dates and indices, which are used for filtering.
   void ClearDatesAndIndices();

   /// \brief Shows/Hides controls for filtering.
   void FilterTypeChanged();

   /// \brief Fills dates and indices for filtering.
   void FillDatesAndIndices();

   /// \brief Used for getting CQGCEL properties values string representation.
   /// \brief If specified value is invalid(see CQGCEL.IsValid() method) then "N/A" is returned.
   /// \param value - variant value
   /// \param valueType - variant type
   /// \param add withMilliseconds - milliseconds to the string, only for VT_DATE
   /// \return String representation of the value.
   CString GetValueAsString(
                  ATL::CComVariant value,
                  VARENUM valueType = VT_EMPTY,
                  bool withMilliseconds = false) const;

   /// \brief Enables/disables controls on the form based on connection status and other parameters.
   void EnableControls();

   /// \brief Shows tag value for specified name.
   void OnEnterEdTagName();

   /// \brief Sets tag value for specified name.
   void OnEnterEdTagValue();

private:

   /// \brief CQGCEL object.
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

   /// \brief Selected condition object.
   ATL::CComPtr<ICQGCondition> m_spCondition;

   /// \brief Resolved condition definitions object.
   ATL::CComPtr<ICQGConditionDefinitions> m_spConditionDefinitions;

private:

   /// \brief Condition count that will be shown in the last chunk list.
   static const long SHOW_COUNT;

   /// \brief Max indices interval for filtering.
   static const long MAX_INTERVAL;

private:

   /// \brief Data connection indicator Label control.
   CStatic m_lblDataConnection;

   /// \brief Request condition definitions Button control.
   CButton m_btnRequestDefinition;

   /// \brief Condition definitions ComboBox control.
   CComboBox m_cmbDefinition;

   /// \brief Definition parameters ListView control.
   CListCtrl m_lvDefParameters;

   /// \brief Active condition ComboBox control.
   CComboBox m_cmbCondition;

   /// \brief Definition name EditBox control.
   CEdit m_edDefinitionName;

   /// \brief Request condition Button control.
   CButton m_btnRequest;

   /// \brief Remove selected condition Button control.
   CButton m_btnRemove;

   /// \brief Remove all condition Button control.
   CButton m_btnRemoveAll;

   /// \brief Last error Label control.
   CStatic m_lblError;

   /// \brief Request status Label control.
   CStatic m_lblStatus;

   /// \brief Show filtered condition Button control.
   CButton m_btnShow;

   /// \brief Tag name EditBox control.
   CEdit m_edTagName;

   /// \brief Tag value EditBox control.
   CEdit m_edTagValue;

   /// \brief All records count Label control.
   CStatic m_lblAllRecsCount;

   /// \brief Filtered records count Label control.
   CStatic m_lblFltrRecsCount;

   /// \brief Last chunk of condition ListView control.
   CListCtrl m_lvLastCondition;

   /// \brief Filtering by indices RadioButton control.
   CButton m_rdIndices;

   /// \brief Filtering by dates RadioButton control.
   CButton m_rdDates;

   /// \brief Filtering type.
   int m_FilterType;

   /// \brief Filtering by indices range start EditBox control.
   CNumEdit m_edStart;

   /// \brief Filtering by indices range start SpinButton control.
   CSpinButtonCtrl m_spnStart;

   /// \brief Filtering by indices range end EditBox control.
   CNumEdit m_edEnd;

   /// \brief Filtering by indices range end SpinButton control.
   CSpinButtonCtrl m_spnEnd;

   /// \brief Filtering by date range start DateTimePicker control.
   CDateTimeCtrl m_dtpStart;

   /// \brief Filtering by date range end DateTimePicker control.
   CDateTimeCtrl m_dtpEnd;

   /// \brief Filtered condition ListView control.
   CListCtrl m_lvCondition;

   /// \brief Request's parameters  ListView control.
   CListCtrl m_lvParameters;

   /// \brief String which describes request definition name.
   CString m_DefinitionName;
   
   /// \brief String which describes request base expression.
   CString m_BaseExpression;

   /// \brief String which describes request range start.
   CString m_RangeStart;

   /// \brief String which describes request range end.
   CString m_RangeEnd;

   /// \brief String which describes request intra day period.
   CString m_IntradayPeriod;

   /// \brief String which describes request historical period.
   CString m_HistoricalPeriod;

   /// \brief String which describes request updates enabled.
   CString m_UpdatesEnabled;

   /// \brief String which describes request include end.
   CString m_IncludeEnd;

   /// \brief String which describes request sessions filter.
   CString m_SessionsFilter;

   /// \brief String which describes request continuation.
   CString m_Continuation;

   /// \brief String which describes request equalize closes.
   CString m_EqualizeCloses;

   /// \brief String which describes request days before expiration.
   CString m_DaysBeforeExpiration;

   /// \brief String which describes request session flags.
   CString m_SessionFlags;

   /// \brief String which describes output range start.
   CString m_OutputStart;

   /// \brief String which describes output range end.
   CString m_OutputEnd;
   
   /// \brief String which describes subscription level.
   CString m_SubscriptionLevel;
   
   /// \brief String which describes recalc period.
   CString m_RecalcPeriod;
};

enum CConditionDlg::eRecordsColumns
{
   rcIndex,
   rcTimestamp,
   rcValue,
};

enum CConditionDlg::eFilterTypes
{
   ftByIndices,
   ftByDates
};

enum CConditionDlg::eDefintionParametersColumns
{
   dpcName,
   dpcComment,
   dpcType,
   dpcDefaultValue
};

enum CConditionDlg::eParametersColumns
{
   pcName,
   pcValue
};